"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tsserverlibrary_1 = require("typescript/lib/tsserverlibrary");
function create(config) {
    return new ToolImpl(config);
}
exports.create = create;
class ToolImpl {
    constructor(config) {
        this.config = config;
        this.config.prefix = this.config.prefix || '&%&%';
        this.config.log = this.config.log || ((msg) => { });
        this.config.actions.forEach(a => { a.commentType = a.commentType || 'line'; });
        this.buildEvalExpr();
    }
    buildEvalExpr() {
        this.allActionsEvalPrefix = '';
        // want to get to an string like this: function moveThisFileTo(path){return {name: 'moveThisFileTo', args: {dest: path} }};
        this.config.actions.forEach(actionConfig => {
            this.allActionsEvalPrefix += `function ${actionConfig.name}(${actionConfig.args.join(', ')}) {return {name: '${actionConfig.name}', args: {${actionConfig.args.join(', ')}}}};\n`;
        });
    }
    /**
     * find all declared actions in given file string
     * @param fileStr content of file where to find action declarations - in general obtained with `sourceFile.getText()`
     */
    findActions(fileStr) {
        return buildUserCalls(fileStr, this.config).map(userCall => {
            try {
                const toEval = this.allActionsEvalPrefix + ';\n' + userCall;
                this.config.log('findActions toEval ' + toEval);
                const result = eval(toEval);
                if (result && typeof result === 'object') {
                    const actionConfig = this.config.actions.find(a => userCall.trim().startsWith(a.name)); // TODO: issue if two start with the same prefix 
                    result.print = actionConfig.print.bind(result);
                    return result;
                }
                else {
                    this.config.log('findActions ignoring eval result because is not an object ' + result);
                }
            }
            catch (ex) {
                this.config.log('findActions eval error ' + ex + ' - ' + ex.stack);
            }
        }).filter(a => a !== undefined);
    }
    getCompletionsAtPosition(fileName, position, options) {
        const completionPrefix = this.config.completionPrefix || 'refactor';
        const result = this.config.actions.map(action => {
            let insertText = [];
            const snippet = typeof action.snippet === 'function' ? action.snippet(fileName, position) : action.snippet;
            if (!snippet) {
                this.config.log('getCompletionsAtPosition aborted because !snippet');
                return;
            }
            const splitted = snippet.split('\n');
            for (let i = 0; i < splitted.length; i++) {
                if (action.commentType === 'line') {
                    insertText.push(`// ${this.config.prefix} ${splitted[i]}`);
                }
                if (action.commentType === 'block') {
                    insertText.push(`\n/* ${this.config.prefix} ${splitted[i]} */\n`);
                }
            }
            const name = `${completionPrefix} ${action.name} ${typeof action.nameExtra === 'function' ? action.nameExtra(fileName, position) : action.nameExtra || ''}`;
            return {
                name,
                kind: tsserverlibrary_1.ScriptElementKind.unknown,
                kindModifiers: tsserverlibrary_1.ScriptElementKindModifier.none,
                sortText: name,
                insertText: insertText.join('\n')
            };
        })
            .filter(a => !!a);
        // this.config.log && this.config.log(`TEXT getCompletionsAtPosition ${result.map(r=>JSON.stringify(r)).join(', ')}`)
        return result;
    }
    getApplicableRefactors(info, refactorName, refactorActionName, fileName, positionOrRange, userPreferences) {
        // const refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange, userPreferences) || []
        const refactors = []; // heads up ! we want only to return the new refactors - user is responsible of concat to prior ones
        const program = info.languageService.getProgram();
        const sourceFile = program.getSourceFile(fileName);
        if (!sourceFile) {
            return { refactors };
        }
        const actions = this.findActions(sourceFile.getText());
        if (!actions || actions.length === 0) {
            return { refactors };
        }
        const selectedAction = actions[0];
        const refactorActions = [{ name: refactorActionName, description: selectedAction.print(selectedAction) }];
        refactors.push({
            name: refactorName,
            description: `${refactorName} description`,
            actions: refactorActions
        });
        return { refactors, selectedAction };
    }
}
function buildUserCalls(fileStr, config) {
    const lines = fileStr.split('\n'); //TODO: use new line format in tsconfig
    let userCalls = [];
    for (let lineIndex = 0; lineIndex < lines.length; lineIndex++) {
        let line = lines[lineIndex].trim();
        let i = line.indexOf(config.prefix);
        if (i === -1) {
            continue;
        }
        let userCall = line.substr(i + config.prefix.length, line.length);
        userCall = userCall.endsWith('*/') ? userCall.substring(0, userCall.length - 3) : userCall;
        for (lineIndex = lineIndex + 1; lineIndex < lines.length; lineIndex++) {
            line = lines[lineIndex];
            const i = line.indexOf(config.prefix);
            if (i === -1) {
                userCalls.push(userCall);
                break;
            }
            userCall += '\n' + line.substr(i + config.prefix.length, line.length);
        }
    }
    return userCalls;
}
exports.buildUserCalls = buildUserCalls;
//# sourceMappingURL=index.js.map